#!/usr/bin/perl -w
###############################################################################
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id: Makefile.pl,v 1.6 2005/05/27 15:03:06 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#################################################################################
# a perl script to check for module requirements, needed by afick
#################################################################################

use strict;
use diagnostics;

#################################################################################
sub check_module($) {
	my $fic = shift(@_);

	open( PROG, '<', $fic ) or warn "impossible to open $fic : $!\n";

	my $nb_ko = 0;
	while (<PROG>) {

		# search for 'use' commands
		# I suppose "require" are used with eval
		if ( $_ =~ m/^use\s+([^\s;(]*)/ ) {

			# get the name without any import list
			my $module = $1;

			#change tree structure to directories
			$module =~ s/::/\//;

			# now test
			eval { require $module . '.pm' };
			if ($@) {
				print "WARNING : module $module not found : $@\n";
				$nb_ko++;
			}

		}
	}
	close(PROG);

	if ( $nb_ko == 0 ) {
		print "FINE : all module founds to install $fic\n";
		return 1;
	}
	else {
		warn
"WARNING : can not install $fic : some necessary modules are missing\n";
		return 0;
	}
}
#################################################################################
# change perl path (shebang) for all perl scripts (*.pl)
sub change_header($$) {
	my $old = shift(@_);
	my $new = shift(@_);

	my @list = glob("*.pl");
	foreach my $fic (@list) {
		print "$fic\n";
		rename $fic, $fic . '.sav';
		open( OLD, '<', $fic . '.sav' );
		open( NEW, '>', $fic );

		my $nbline = 0;
		while (<OLD>) {
			if ( $nbline == 0 ) {
				print NEW '#!' . $new . " -w\n";
			}
			else {
				print NEW $_;
			}
			$nbline++;
		}

		close(OLD);
		close(NEW);
	}
	chmod 0755, @list;
}
#################################################################################

# search and configure perl path
################################
# use which : a very common unix command
# perhaps it will be better to use the "type" builtin
my $cmd      = "which perl";
my @out      = `$cmd`;
my $perlpath = $out[0];
chomp($perlpath);

if ( $perlpath ne '/usr/bin/perl' ) {
	change_header( '/usr/bin/perl', $perlpath );
}

# check for needed modules
# ################################
if ( check_module('afick.pl') ) {
	rename( 'Makefile.in', 'makefile' );
	if ( check_module('afick-tk.pl') ) {

		# all is ok
		print "now you can run 'make all' to install console and gui\n";
	}
	else {

		# gui can be installed
		print "now you can run 'make install' to install console tool\n";
	}
}
else {

	# no install possible
	warn "you can not install any afick component\n";
}
